/* ginit.c -- Graphics initialization/restoration module
 *
 *	TC2 CGA BGI bug with cleardevice() fixed.
 *	Cursor made smaller for speed.
 *	Ability to select Color via "kd.ini" file.
 *		Izumi Ohzawa, 3-2-90
 *
 *      Written to be compiled with Turbo C 2.0 (or later presumably) by
 *		Izumi Ohzawa, 1-8-90
 *		Group in Neurobiology/School of Optometry
 *		University of California, Berkeley
 *		Berkeley, CA 94720
 *		izumi@violet.berkeley.edu
 *
 *	This program/code is hereby released into the public domain.
 *	No copyright is claimed, and no restriction is imposed for any use,
 *	commercial or otherwise.  Use it at your own risk.
 * ------------------------------------------------------------------------ */

/* #define DEBUG 1 */

#include <stdio.h>
#include <dos.h>	/* for int86() */
#include <stdlib.h>
#include <alloc.h>
#include <graphics.h>
#include <string.h>
#include "kjis.h"
#include "prototyp.h"

static unsigned int far *vidpix;  /* # of pixels per text line in graphics mode */
static unsigned int vpsave;	  /* Save location for tampered BIOS param */

/* === This works only with EGA/VGA.  This is to set line spacing to
 *	a desired value (in pixels) for video BIOS scroll functions.
 * ................................................................... */
void
Tamper_BIOS_data(void)
{
	vidpix = MK_FP(0x40, 0x85);  /* ptr to BIOS parm.  Bad thing, I know */
#ifdef DEBUG
	printf("Pixels/Row = %d\n\n", *vidpix);
	getch();
#endif
	vpsave = *vidpix;	/* save current pixels/line (font height) */
	*vidpix = PixelPerRow;	/* set new pixels/line for Kanji */
}

void
Restore_BIOS_data(void)
{
	*vidpix = vpsave;	/* restore tampered BIOS paremter */
}


/* === Clear the entire area of the screen.  Fix needed because
**	cleardevice() does not work with CGA.BGI */
void
Clear_All(void)
{
	if(GraphDriver != CGA)	/* Not CGA */
		cleardevice();
	else			/* is CGA -- Workaround for Turbo C 2.0 bug */
	{
		setviewport(0, 0, MaxX, MaxY, 0);
		clearviewport();
	}
}


/* === Initialize Graphics state for our use */
int
Initialize_Graphics(char *BGIpath)
{
int errcode;
        /* Redirect Video BIOS from 0x10 to 0x68 to prevent re-entry
         * into interrupt function in kd.c */
        setvect(NEWVIDBIOS, getvect(VIDBIOS));

	detectgraph(&GraphDriver, &GraphMode);
	if(GraphDriver < 0)
	{
	    fprintf(stderr,"No graphics hardware detected!\n");
	    return(1);
	}

	ColorOK = 0;
	errcode = 0;
	switch(GraphDriver)
	{
	   case MCGA:
		   GraphMode = MCGAMED;	/* 640x200 to make it as CGA */
	   case CGA:
		   ScrollMode = BIOS;	/* BIOS scroll */
                   ScrollMult = 2;      /* 2 lines of 8 pixel high */
		   PixelPerRow = 16;	
		   DefAscColor = AscColor = 1;
		   break;
	   case EGA64:
		   GraphMode = EGA64HI;
	   case EGAMONO:
		   DefAscColor = AscColor = 7;
		   ScrollMode = BIOS;	/* 1 line BIOS scroll */
		   OK2MessBIOS = 1;	/* BIOS data area OK */
		   break;
	   case EGA:
	   case VGA:
		   ColorOK = 1;				/* OK to use color */
		   ScrollMode = BIOS;
		   OK2MessBIOS = 1;	/* BIOS data area OK */
		   break;
	   case HERCMONO:
		   ScrollMode = MMOVE;
		   PixelPerRow = 16;	/* for scroll by memmove */
		   DefAscColor = AscColor = 1;
		   break;
	   default:
		   ScrollMode = MMOVE;	/* Be safe. BIOS supports scroll ?? */
		   DefAscColor = AscColor = 7;
		   errcode = 1;
		   break;
	}  /* end of switch(GraphDriver) */

	if(errcode)
	{
	    fprintf(stderr,"Graphics Adapter is not VGA, EGA, CGA, MCGA or Hercules.\n");
	    fprintf(stderr,"Program can probably be made to run, but the author did\n");
	    fprintf(stderr,"not have access to the technical info. on this adapter.\n");
	    fprintf(stderr,"GraphDriver= %d,  GraphMode= %d\n",
			GraphDriver, GraphMode);
	    return(2);
	}

        initgraph(&GraphDriver, &GraphMode, BGIpath);
	errcode = graphresult();
	if(errcode <0)
	{
	    fprintf(stderr,"initgraph() error: %s\n", grapherrormsg(errcode));
	    if(errcode == -3)
		fprintf(stderr,"Copy *.BGI files to directory: %s\n", BGIpath);
	    return(3);
	}

	MaxX = getmaxx();
	MaxY = getmaxy();
	NumColumn = (MaxX+1)/8;		/* 8 -- ASCII char width */
	MaxColumn = NumColumn -1;
        NumRow = (MaxY+1)/PixelPerRow;
        MaxRow = NumRow -1;

	if(ColorOK)			/* set cursor color */
	    setcolor(CursorColor);
	else
	    setcolor(AscColor);
	Cursor_Bitmap = malloc(imagesize(0,7,7,7));	/* allocate mem for cursor */
	outtextxy(0, 0, "_");				/* put cursor char */
	getimage(0, 7, 7, 7, Cursor_Bitmap);		/* get cursor image */
	lastxpos = 0;
	lastypos = PixelPerRow-1;
	Set_Scroll_Area(0, 0, MaxColumn, MaxRow);
        setcolor(AscColor);	/* set ASCII mode color */

#ifdef DEBUG
	printf("x:y = %d:%d\n", getmaxx(), getmaxy());
	getch();
#endif
	if(OK2MessBIOS)
	    Tamper_BIOS_data();	/* Gotta do it to scroll in graphics mode */
	Clear_All();
	putimage(lastxpos, lastypos, Cursor_Bitmap, XOR_PUT);	/* put first cursor */
	return(0);	/* OK */
}

int
Exit_Cleanup(void)
{
int boxbgcolor = 0;
int boxframecolor = 7;
int msgcolor = 7;
	if(ColorOK)
	{
		boxbgcolor = BLUE;
		boxframecolor = LIGHTRED;
		msgcolor = WHITE;
	}

	if(OK2MessBIOS)
	    Restore_BIOS_data();
	setfillstyle(SOLID_FILL, boxbgcolor);
	bar(20, 10, 300, 40);
	setlinestyle(SOLID_LINE, 0xffff, NORM_WIDTH);
	setcolor(boxframecolor);
	rectangle(20, 10, 300, 40);
	setcolor(msgcolor);
	outtextxy(40, 22, "*** PRESS ANY KEY TO EXIT ***");
	while( kbhit()) getch();	/* drain accumulated chars */
	getch();
        closegraph();
	return(0);
}


