/* read-ini.c -- routine to read in kd.ini file for configration 
 *		Izumi Ohzawa, 2-12-90
 *		Group in Neurobiology/School of Optometry
 *		University of California, Berkeley
 *		Berkeley, CA 94720
 *		izumi@violet.berkeley.edu
 *
 *	This program/code is hereby released into the public domain.
 *	No copyright is claimed, and no restriction is imposed for any use,
 *	commercial or otherwise.  Use at your own risk.
 * ------------------------------------------------------------------------- */

#include <stdio.h>
#include <string.h>
#include "kjis.h"

static char *inifilename = "\\kd.ini";	/* Initialization file */

/* ===== List of parameter names in especfile .. For reading ============= */
static char *parname[] =
	{
		"ASCCOLOR", "KANJICOLOR", "BACKCOLOR", "CURSORCOLOR",
		"LINESPACING", "LEVEL2FONT", "EMPHASISCOLOR", "BOLDCOLOR",
		"REVCHARCOLOR", "REVBGCOLOR", "SETSCROLLAREA",
		""
	};
#define NKEYWORD	  11		/* # of keywords in *parname */

/* This routine will look for initialization file "kd.ini" in the
**	directory from which "kd.exe" is loaded (passed as argument *loaddir)
** If the file does not exit, default values hard-coded (mostly in kjis.h)
**	are used.  Kd.ini file needs to contain only those parameters you
**	want to override.
*/
int
Read_Ini_File(char *loaddir)
{
int i, prmnumber, iy;
FILE *fpin;
char linebuf[128];
char strbuf1[32];

	strcpy(linebuf, loaddir);
	strcat(linebuf, inifilename);
	fpin = fopen(linebuf, "r");
	if (fpin == NULL)
		return(1);	/* use hard-coded default values */

	/* OK file opened, read */
	fprintf(stderr, "Reading INI file... ");
	while( ( fgets(linebuf, 120, fpin) ) != NULL )
	{
	    strbuf1[0] = '\0';		/* clear keyword buffer */
	    if(linebuf[0] == '#' || linebuf[0] == ';')
		continue;		/* ignore comments */
	    sscanf( linebuf, "%s", strbuf1);	/* first word */
	    if(strlen(strbuf1) == 0)
		continue;		/* empty line */
	    strupr(strbuf1);
	    prmnumber = -1;
	    /* Do linear search in the keyword list */
	    for(i=0; i<NKEYWORD; i++)
	    {
		   if( strcmp( strbuf1, parname[i]) == 0 )
		   {
			prmnumber = i;
			break;
		   }
	    }

	    /* --- lookup in *parname[] done ----- */
	    if( prmnumber == -1 )
	    {
		fprintf(stderr, "...Unknown keyword \"%s\" in INI file.\n", strbuf1);
	    }
	    else    /* valid prmnumber .. param name found */
	    {
		switch(prmnumber)
		{
		   case 0:	/* AscColor */
			sscanf(linebuf, "%*s %d", &DefAscColor);
			AscColor = DefAscColor;
			break;
		   case 1:	/* KanjiColor */
			sscanf(linebuf, "%*s %d", &KanjiColor);
			break;
		   case 2:	/* BackColor */
			sscanf(linebuf, "%*s %d", &BackColor);
			break;	
		   case 3:	/* CursorColor */
			sscanf(linebuf, "%*s %d", &CursorColor);
			break;
		   case 4:	/* LineSpacing */
			sscanf(linebuf, "%*s %d", &PixelPerRow);
			if(PixelPerRow < 16) PixelPerRow = 16;
			break;
		   case 5:	/* Level2Font */
			sscanf(linebuf, "%*s %s", strbuf1);
			strupr(strbuf1);
			iy = strbuf1[0];
			if(iy == 'R')
				Level2Font = JIS2_RAM; 	/* RAM */
			else if(iy == 'F' || iy == 'D')
				Level2Font = JIS2_FILE;	/* File or Disk */
			else    Level2Font = JIS2_NONE;	/* None, level-1 only */			
			break;
		   case 6:	/* EmphasisColor */
			sscanf(linebuf, "%*s %d", &EmphasisColor);
			break;
		   case 7:	/* BoldColor */
			sscanf(linebuf, "%*s %d", &BoldColor);
			break;
		   case 8:	/* RevCharColor */
			sscanf(linebuf, "%*s %d", &RevCharColor);
			break;
		   case 9:	/* RevBGColor */
			sscanf(linebuf, "%*s %d", &RevBGColor);
			break;
		   case 10:	/* SetScrollArea */
			sscanf(linebuf, "%*s %d", &SetScrollArea);
			break;

		   default:
			break;
		 }   /* end of switch() */
	    }        /* end of else */
	}   /* end of while( (fgets .. )) loop */
	fprintf(stderr, " DONE\n");
	fclose(fpin);
	return(0);
}

